#!/usr/bin/perl -w
use strict;
#use cmdline_args;

if (!@ARGV || $ARGV[0] eq "help") {
    print STDERR <<EOF;

    Script to make a crude conversion of hapmap format files that come out of
    TASSEL to VCF format in order to load in ld-impute. This script is only
    intended as a stop-gap, ld-impute needs to be a little bit redesigned and
    have a native method to input hapmap format as well as VCF directly.

    Usage: -s <fake snp quality> -g <fake genotype quality> 

    Reads list of hapmap format files on STDIN, writes combined data as VCF
    to STDOUT.

EOF
exit -1;
}

my ($snp_q, $genotype_q) = (60, 20);
my %args;
$args{'-s'} = [ \$snp_q, 1 ];
$args{'-g'} = [ \$genotype_q, 1 ];
get_options(\%args, \@ARGV);


my $n_files = 0;
my $sample_map;
while(<STDIN>) {
    chomp;
    next if m/^#/;
    next if m/^\s*$/;
    $n_files++;

    my ($input_fname) = split/\t/;

    open F, "<$input_fname"
	or die "Can't open hapmap input file $input_fname ($!)";

    my $header_line = <F>;
    chomp $header_line;

    if ($n_files == 1) {
	$sample_map = assign_samples($header_line);
    }
    my @column_headers = split/\t/,$header_line;
    my @column_map;
    for(my $i=11;$i<@column_headers;$i++) {
	my ($sample_name) = split/:/,$column_headers[$i];
	if (!defined($sample_map->{$sample_name})) {
	    printf STDERR "Sample %s in input file %s was not defined in the ".
		"first input file\n", $sample_name, $input_fname;
	    $column_map[$i] = -1;
	} else {
	    $column_map[$i] = $sample_map->{$sample_name};
	}
    }

    my @output_cols;
    for(my $i=0;$i<keys(%{$sample_map});$i++) {
	$output_cols[$i] = "./.:0";
    }

    while(<F>) {
	chomp;
	
	my (@columns) = split/\t/;

	my ($a, $b) = split/\//,$columns[1],2;

	my ($nAA, $nAB, $nBB, $n_missing) = (0, 0, 0, 0);
	for(my $i=11;$i<@columns;$i++) {
	    next if $column_map[$i] == -1;
	    if ($columns[$i] eq "N") {
		$output_cols[$column_map[$i]] = "./.:0";
		$n_missing++;
	    } elsif ($columns[$i] eq $a) {
		$output_cols[$column_map[$i]] = "0/0:$genotype_q";
		$nAA++;
	    } elsif ($columns[$i] eq $b) {
		$output_cols[$column_map[$i]] = "1/1:$genotype_q";
		$nBB++;
	    } else {
		$output_cols[$column_map[$i]] = "0/1:$genotype_q";
		$nAB++;
	    }
	}	    
	
	print join("\t", $columns[2], $columns[3], $columns[0], $a, $b, $snp_q,
		   ".", "NAA=$nAA;NAB=$nAB;NBB=$nBB;NM=$n_missing", "GT:GQ",
		   @output_cols),"\n";
    }
    close F;

}

sub assign_samples {
    my ($header_line) = @_;
    
    chomp $header_line;

    my (undef, undef, undef, undef, undef, undef, undef, undef, undef, undef,
	undef, @sample_names) = split/\t/,$header_line;
    
    my %map;
    my $output_index = 0;
    my @samples;
    for my $column_header ( @sample_names ) {
	my ($sample_name) = split/:/,$column_header;
	
	$map{$sample_name} = $output_index++;
	print "##SAMPLE=<ID=$sample_name,SUBPOP=ALL>\n";
	push @samples, $sample_name;
    }
    print "#CHROM\tPOS\tID\tREF\tALT\tQUAL\tFILTER\tINFO\tFORMAT\t",
    join("\t",@samples),"\n";

    return \%map;
}
    

# copied out of cmdline_args.pm so we don't have to have that module around
# in a separate file and deal with perl library path settings
sub get_options {
    my ($validopt_ref, $argv_ref) = @_;
    my ($option, $error);

    if (! @{$argv_ref}) {
	print "No command line options\n";
	return 0;
    }

    while(@{$argv_ref}>0) {
	$option = shift @{$argv_ref};

	if (! defined($validopt_ref->{$option})) {
	    print STDERR "Unknown option \"$option\"\n";
	    $error = 1;
	    next;
	}

	if ($validopt_ref->{$option}->[1]) {
	    # Option which requires argument, shift in the next argument from
	    # the command line
	    ${$validopt_ref->{$option}->[0]} = shift @{$argv_ref};
	} else {
	    # Boolean flag: invert default value
	    if (${$validopt_ref->{$option}->[0]}) {
	         ${$validopt_ref->{$option}->[0]} = 0;
            } else {
                 ${$validopt_ref->{$option}->[0]} = 1;
            }
        }
    }

    return $error;
}
